@@ -1,6 +1,10 @@
 from .crossval import VectorClassificationModelCrossValidator, VectorRegressionModelCrossValidator, \
     VectorClassificationModelCrossValidationData, VectorRegressionModelCrossValidationData
 from .eval_util import RegressionEvaluationUtil, ClassificationEvaluationUtil, MultiDataEvaluationUtil, \
-    evalModelViaEvaluator
+    evalModelViaEvaluator, createEvaluationUtil, createVectorModelEvaluator, createVectorModelCrossValidator
 from .evaluator import VectorClassificationModelEvaluator, VectorRegressionModelEvaluator, \
-    VectorRegressionModelEvaluationData, VectorClassificationModelEvaluationData
+    VectorRegressionModelEvaluationData, VectorClassificationModelEvaluationData, \
+    RuleBasedVectorClassificationModelEvaluator, RuleBasedVectorRegressionModelEvaluator
+
+# imports required for backward compatibility
+from ..data import DataSplitter, DataSplitterFractional
