@@ -13,10 +13,11 @@ from .torch_data import TensorScaler, VectorDataUtil, ClassificationVectorDataUt
     TorchDataSetProviderFromDataUtil, TorchDataSetProvider, Tensoriser, TorchDataSetFromDataFrames
 from .torch_enums import ClassificationOutputMode
 from .torch_opt import NNOptimiser, NNLossEvaluatorRegression, NNLossEvaluatorClassification, NNOptimiserParams, TrainingInfo
+from ..data import DataFrameSplitter
 from ..normalisation import NormalisationMode
 from ..util.dtype import toFloatArray
-from ..util.string import objectRepr, ToStringMixin
-from ..vector_model import VectorRegressionModel, VectorClassificationModel
+from ..util.string import ToStringMixin
+from ..vector_model import VectorRegressionModel, VectorClassificationModel, TrainingContext
 
 log: logging.Logger = logging.getLogger(__name__)
 
@@ -96,6 +97,8 @@ class TorchModel(ABC, ToStringMixin):
     """
     log: logging.Logger = log.getChild(__qualname__)
 
+    NORMALISATION_CHECK_THRESHOLD = 5
+
     def __init__(self, cuda=True) -> None:
         self.cuda: bool = cuda
         self.module: Optional[torch.nn.Module] = None
@@ -229,9 +232,14 @@ class TorchModel(ABC, ToStringMixin):
         if createBatch:
             inputs = [t.view(1, *X.size()) for t in inputs]
 
-        maxValue = max([t.max().item() for t in inputs])
-        if maxValue > 2:
-            log.warning("Received input which is likely to not be correctly normalised: maximum value in input tensor is %f" % maxValue)
+        # check input normalisation
+        if self.NORMALISATION_CHECK_THRESHOLD is not None:
+            maxValue = 0.0
+            for t in inputs:
+                if t.is_floating_point():  # ignore any integer tensors (which typically contain lengths)
+                    maxValue = max(t.abs().max().item(), maxValue)
+            if maxValue > self.NORMALISATION_CHECK_THRESHOLD:
+                log.warning("Received input which is likely to not be correctly normalised: maximum abs. value in input tensor is %f" % maxValue)
 
         if mcDropoutSamples is None:
             y = model(*inputs)
@@ -381,11 +389,15 @@ class TorchVectorRegressionModel(VectorRegressionModel):
         self.modelKwArgs = modelKwArgs
         self.model: Optional[TorchModel] = None
         self.inputTensoriser: Optional[Tensoriser] = None
+        self.torchDataSetProviderFactory: Optional[TorchDataSetProviderFactory] = None
+        self.dataFrameSplitter: Optional[DataFrameSplitter] = None
 
     def __setstate__(self, state) -> None:
         state["nnOptimiserParams"] = NNOptimiserParams.fromDictOrInstance(state["nnOptimiserParams"])
-        if "inputTensoriser" not in state:
-            state["inputTensoriser"] = None
+        newOptionalMembers = ["inputTensoriser", "torchDataSetProviderFactory", "dataFrameSplitter"]
+        for m in newOptionalMembers:
+            if m not in state:
+                state[m] = None
         s = super()
         if hasattr(s, '__setstate__'):
             s.__setstate__(state)
@@ -396,17 +408,27 @@ class TorchVectorRegressionModel(VectorRegressionModel):
         self.inputTensoriser = tensoriser
         return self
 
+    def withTorchDataSetProviderFactory(self, torchDataSetProviderFactory: "TorchDataSetProviderFactory") -> __qualname__:
+        self.torchDataSetProviderFactory = torchDataSetProviderFactory
+        return self
+
+    def withDataFrameSplitter(self, dataFrameSplitter: DataFrameSplitter) -> __qualname__:
+        self.dataFrameSplitter = dataFrameSplitter
+        return self
+
     def _createTorchModel(self) -> TorchModel:
         return self.modelClass(*self.modelArgs, **self.modelKwArgs)
 
     def _createDataSetProvider(self, inputs: pd.DataFrame, outputs: pd.DataFrame) -> TorchDataSetProvider:
-        dataUtil = VectorDataUtil(inputs, outputs, self.model.cuda, normalisationMode=self.normalisationMode, inputTensoriser=self.inputTensoriser)
-        return TorchDataSetProviderFromDataUtil(dataUtil, self.model.cuda)
+        factory = self.torchDataSetProviderFactory
+        if factory is None:
+            factory = DefaultRegressionTorchDataSetProviderFactory()
+        return factory.createDataSetProvider(inputs, outputs, self, self._trainingContext, self.inputTensoriser, self.dataFrameSplitter)
 
     def _fit(self, inputs: pd.DataFrame, outputs: pd.DataFrame) -> None:
         if self.inputTensoriser is not None:
             log.info(f"Fitting {self.inputTensoriser} ...")
-            self.inputTensoriser.fit(inputs)
+            self.inputTensoriser.fit(inputs, model=self)
         self.model = self._createTorchModel()
         dataSetProvider = self._createDataSetProvider(inputs, outputs)
         self.model.fit(dataSetProvider, self.nnOptimiserParams)
@@ -463,12 +485,16 @@ class TorchVectorClassificationModel(VectorClassificationModel):
         self.modelArgs = modelArgs
         self.modelKwArgs = modelKwArgs
         self.model: Optional[VectorTorchModel] = None
-        self.inputTensoriser = None
+        self.inputTensoriser: Optional[Tensoriser] = None
+        self.torchDataSetProviderFactory: Optional[TorchDataSetProviderFactory] = None
+        self.dataFrameSplitter: Optional[DataFrameSplitter] = None
 
     def __setstate__(self, state) -> None:
         state["nnOptimiserParams"] = NNOptimiserParams.fromDictOrInstance(state["nnOptimiserParams"])
-        if "inputTensoriser" not in state:
-            state["inputTensoriser"] = None
+        newOptionalMembers = ["inputTensoriser", "torchDataSetProviderFactory", "dataFrameSplitter"]
+        for m in newOptionalMembers:
+            if m not in state:
+                state[m] = None
         if "outputMode" not in state:
             state["outputMode"] = ClassificationOutputMode.PROBABILITIES
         s = super()
@@ -481,13 +507,22 @@ class TorchVectorClassificationModel(VectorClassificationModel):
         self.inputTensoriser = tensoriser
         return self
 
+    def withTorchDataSetProviderFactory(self, torchDataSetProviderFactory: "TorchDataSetProviderFactory") -> __qualname__:
+        self.torchDataSetProviderFactory = torchDataSetProviderFactory
+        return self
+
+    def withDataFrameSplitter(self, dataFrameSplitter: DataFrameSplitter) -> __qualname__:
+        self.dataFrameSplitter = dataFrameSplitter
+        return self
+
     def _createTorchModel(self) -> VectorTorchModel:
         return self.modelClass(*self.modelArgs, **self.modelKwArgs)
 
     def _createDataSetProvider(self, inputs: pd.DataFrame, outputs: pd.DataFrame) -> TorchDataSetProvider:
-        dataUtil = ClassificationVectorDataUtil(inputs, outputs, self.model.cuda, len(self._labels),
-            normalisationMode=self.normalisationMode, inputTensoriser=self.inputTensoriser)
-        return TorchDataSetProviderFromDataUtil(dataUtil, self.model.cuda)
+        factory = self.torchDataSetProviderFactory
+        if factory is None:
+            factory = DefaultClassificationTorchDataSetProviderFactory()
+        return factory.createDataSetProvider(inputs, outputs, self, self._trainingContext, self.inputTensoriser, self.dataFrameSplitter)
 
     def _fitClassifier(self, inputs: pd.DataFrame, outputs: pd.DataFrame) -> None:
         if len(outputs.columns) != 1:
@@ -495,7 +530,7 @@ class TorchVectorClassificationModel(VectorClassificationModel):
 
         if self.inputTensoriser is not None:
             log.info(f"Fitting {self.inputTensoriser} ...")
-            self.inputTensoriser.fit(inputs)
+            self.inputTensoriser.fit(inputs, model=self)
 
         # transform outputs: for each data point, the new output shall be the index in the list of labels
         labels: pd.Series = outputs.iloc[:, 0]
@@ -528,3 +563,29 @@ class TorchVectorClassificationModel(VectorClassificationModel):
 
     def _toStringExcludes(self) -> List[str]:
         return super()._toStringExcludes() + ["modelClass", "modelArgs", "modelKwArgs", "inputTensoriser"]
+
+
+class TorchDataSetProviderFactory(ABC):
+    @abstractmethod
+    def createDataSetProvider(self, inputs: pd.DataFrame, outputs: pd.DataFrame,
+            model: Union[TorchVectorRegressionModel, TorchVectorClassificationModel], trainingContext: TrainingContext,
+            inputTensoriser: Optional[Tensoriser], dataFrameSplitter: Optional[DataFrameSplitter]) -> TorchDataSetProvider:
+        pass
+
+
+class DefaultClassificationTorchDataSetProviderFactory(TorchDataSetProviderFactory):
+    def createDataSetProvider(self, inputs: pd.DataFrame, outputs: pd.DataFrame, model: TorchVectorClassificationModel,
+            trainingContext: TrainingContext, inputTensoriser: Optional[Tensoriser],
+            dataFrameSplitter: Optional[DataFrameSplitter]) -> TorchDataSetProvider:
+        dataUtil = ClassificationVectorDataUtil(inputs, outputs, model.model.cuda, len(model._labels),
+            normalisationMode=model.normalisationMode, inputTensoriser=inputTensoriser, dataFrameSplitter=dataFrameSplitter)
+        return TorchDataSetProviderFromDataUtil(dataUtil, model.model.cuda)
+
+
+class DefaultRegressionTorchDataSetProviderFactory(TorchDataSetProviderFactory):
+    def createDataSetProvider(self, inputs: pd.DataFrame, outputs: pd.DataFrame, model: TorchVectorRegressionModel,
+            trainingContext: TrainingContext, inputTensoriser: Optional[Tensoriser],
+            dataFrameSplitter: Optional[DataFrameSplitter]) -> TorchDataSetProvider:
+        dataUtil = VectorDataUtil(inputs, outputs, model.model.cuda, normalisationMode=model.normalisationMode,
+            inputTensoriser=inputTensoriser, dataFrameSplitter=dataFrameSplitter)
+        return TorchDataSetProviderFromDataUtil(dataUtil, model.model.cuda)
