@@ -7,6 +7,7 @@ import numpy as np
 import torch
 
 from .. import normalisation
+from ..data import DataFrameSplitter, DataFrameSplitterFractional
 from ..util.dtype import toFloatArray
 
 
@@ -115,7 +116,12 @@ class Tensoriser(ABC):
         pass
 
     @abstractmethod
-    def fit(self, df: pd.DataFrame):
+    def fit(self, df: pd.DataFrame, model=None):
+        """
+        :param df: the data frame with which to fit this tensoriser
+        :param model: the model in the context of which the fitting takes place (if any).
+            The fitting process may set parameters within the model that can only be determined from the (pre-tensorised) data.
+        """
         pass
 
 
@@ -123,7 +129,7 @@ class RuleBasedTensoriser(Tensoriser, ABC):
     """
     Base class for tensorisers which transforms data frames into tensors based on a predefined set of rules and does not require fitting
     """
-    def fit(self, df: pd.DataFrame):
+    def fit(self, df: pd.DataFrame, model=None):
         pass
 
 
@@ -187,7 +193,8 @@ class DataUtil(ABC):
 
 class VectorDataUtil(DataUtil):
     def __init__(self, inputs: pd.DataFrame, outputs: pd.DataFrame, cuda: bool, normalisationMode=normalisation.NormalisationMode.NONE,
-            differingOutputNormalisationMode=None, inputTensoriser: Optional[Tensoriser] = None, outputTensoriser: Optional[Tensoriser] = None):
+            differingOutputNormalisationMode=None, inputTensoriser: Optional[Tensoriser] = None, outputTensoriser: Optional[Tensoriser] = None,
+            dataFrameSplitter: Optional[DataFrameSplitter] = None):
         """
         :param inputs: the data frame of inputs
         :param outputs: the data frame of outputs
@@ -206,6 +213,7 @@ class VectorDataUtil(DataUtil):
         self.inputTensorScaler = TensorScalerFromVectorDataScaler(self.inputVectorDataScaler, cuda)
         self.outputVectorDataScaler = normalisation.VectorDataScaler(self.outputs, normalisationMode if differingOutputNormalisationMode is None else differingOutputNormalisationMode)
         self.outputTensorScaler = TensorScalerFromVectorDataScaler(self.outputVectorDataScaler, cuda)
+        self.dataFrameSplitter = dataFrameSplitter
 
     def getOutputTensorScaler(self):
         return self.outputTensorScaler
@@ -214,11 +222,15 @@ class VectorDataUtil(DataUtil):
         return self.inputTensorScaler
 
     def splitInputOutputPairs(self, fractionalSizeOfFirstSet):
-        n = self.inputs.shape[0]
-        sizeA = int(n * fractionalSizeOfFirstSet)
-        indices = list(range(n))
-        indices_A = indices[:sizeA]
-        indices_B = indices[sizeA:]
+        splitter = self.dataFrameSplitter
+        if splitter is None:
+            # By default, we use a simple fractional split without shuffling.
+            # Shuffling is usually unnecessary, because in evaluation contexts, the data may have already been shuffled by the evaluator
+            # (unless explicitly disabled by the user). Furthermore, not shuffling gives the user the possibility to manually
+            # order the data in ways that result in desirable fractional splits (though the user may, of course, simply override
+            # the splitter to achieve any desired split).
+            splitter = DataFrameSplitterFractional(shuffle=False)
+        indices_A, indices_B = splitter.computeSplitIndices(self.inputs, fractionalSizeOfFirstSet)
         A = self._inputOutputPairs(indices_A)
         B = self._inputOutputPairs(indices_B)
         return A, B
@@ -250,12 +262,13 @@ class VectorDataUtil(DataUtil):
 
 class ClassificationVectorDataUtil(VectorDataUtil):
     def __init__(self, inputs: pd.DataFrame, outputs: pd.DataFrame, cuda, numClasses, normalisationMode=normalisation.NormalisationMode.NONE,
-            inputTensoriser: Tensoriser = None, outputTensoriser: Tensoriser = None):
+            inputTensoriser: Tensoriser = None, outputTensoriser: Tensoriser = None, dataFrameSplitter: Optional[DataFrameSplitter] = None):
         if len(outputs.columns) != 1:
             raise Exception(f"Exactly one output dimension (the class index) is required, got {len(outputs.columns)}")
         super().__init__(inputs, outputs, cuda, normalisationMode=normalisationMode,
             differingOutputNormalisationMode=normalisation.NormalisationMode.NONE, inputTensoriser=inputTensoriser,
-            outputTensoriser=TensoriserClassLabelIndices() if outputTensoriser is None else outputTensoriser)
+            outputTensoriser=TensoriserClassLabelIndices() if outputTensoriser is None else outputTensoriser,
+            dataFrameSplitter=dataFrameSplitter)
         self.numClasses = numClasses
 
     def modelOutputDim(self):
