@@ -13,6 +13,7 @@ from typing import Any, Callable, Iterator, List, Optional, TypeVar
 
 import joblib
 
+from .hash import pickleHash
 from .pickle import PickleFailureDebugger
 
 log = logging.getLogger(__name__)
@@ -95,11 +96,15 @@ class DelayedUpdateHook:
     Ensures that a given function is executed after an update happens, but delay the execution until
     there are no further updates for a certain time period
     """
-    def __init__(self, fn: Callable[[], Any], timePeriodSecs):
+    def __init__(self, fn: Callable[[], Any], timePeriodSecs, periodicallyExecutedFn: Optional[Callable[[], Any]] = None):
         """
         :param fn: the function to eventually call after an update
         :param timePeriodSecs: the time that must pass while not receiving further updates for fn to be called
+        :param periodicallyExecutedFn: a function to execute periodically (every timePeriodSecs seconds) in the busy waiting loop,
+            which may, for example, log information or apply additional executions, which must not interfere with the correctness of
+            the execution of fn
         """
+        self.periodicallyExecutedFn = periodicallyExecutedFn
         self.fn = fn
         self.timePeriodSecs = timePeriodSecs
         self._lastUpdateTime = None
@@ -117,6 +122,8 @@ class DelayedUpdateHook:
             while True:
                 time.sleep(self.timePeriodSecs)
                 timePassedSinceLastUpdate = time.time() - self._lastUpdateTime
+                if self.periodicallyExecutedFn is not None:
+                    self.periodicallyExecutedFn()
                 if timePassedSinceLastUpdate >= self.timePeriodSecs:
                     self.fn()
                     return
@@ -129,6 +136,68 @@ class DelayedUpdateHook:
             self._threadLock.release()
 
 
+class PeriodicUpdateHook:
+    """
+    Periodically checks whether a function shall be called as a result of an update, the function potentially
+    being non-atomic (i.e. it may take a long time to execute such that new updates may come in while it is
+    executing). Two function all mechanisms are in place:
+
+        * a function which is called if there has not been a new update for a certain time period (which may be called
+          several times if updates come in while the function is being executed)
+        * a function which is called periodically
+
+    """
+    def __init__(self, checkIntervalSecs: float, noUpdateTimePeriodSecs: float = None, noUpdateFn: Callable[[], Any] = None,
+            periodicFn: Optional[Callable[[], Any]] = None):
+        """
+        :param checkIntervalSecs: the time period, in seconds, between checks
+        :param noUpdateTimePeriodSecs: the time period after which to execute noUpdateFn if no further updates have come in.
+            This must be at least as large as checkIntervalSecs. If None, use checkIntervalSecs.
+        :param noUpdateFn: the function to call if there have been no further updates for noUpdateTimePeriodSecs seconds
+        :param periodicFn: a function to execute periodically (every checkIntervalSecs seconds) in the busy waiting loop,
+            which may, for example, log information or apply additional executions, which must not interfere with the correctness of
+            the execution of fn
+        """
+        if noUpdateTimePeriodSecs is None:
+            noUpdateTimePeriodSecs = checkIntervalSecs
+        elif noUpdateTimePeriodSecs < checkIntervalSecs:
+            raise ValueError("noUpdateTimePeriodSecs must be at least as large as checkIntervalSecs")
+        self._periodicFn = periodicFn
+        self._checkIntervalSecs = checkIntervalSecs
+        self._noUpdateTimePeriodSecs = noUpdateTimePeriodSecs
+        self._noUpdateFn = noUpdateFn
+        self._lastUpdateTime = None
+        self._thread = None
+        self._threadLock = threading.Lock()
+
+    def handleUpdate(self):
+        """
+        Notifies of an update, making sure the functions given at construction will be called as specified
+        """
+        self._lastUpdateTime = time.time()
+
+        def doPeriodicCheck():
+            while True:
+                time.sleep(self._checkIntervalSecs)
+                checkTime = time.time()
+                if self._periodicFn is not None:
+                    self._periodicFn()
+                timePassedSinceLastUpdate = checkTime - self._lastUpdateTime
+                if timePassedSinceLastUpdate >= self._noUpdateTimePeriodSecs:
+                    if self._noUpdateFn is not None:
+                        self._noUpdateFn()
+                    # if no further updates have come in, we terminate the thread
+                    if self._lastUpdateTime < checkTime:
+                        return
+
+        if self._thread is None or not self._thread.is_alive():
+            self._threadLock.acquire()
+            if self._thread is None or not self._thread.is_alive():
+                self._thread = threading.Thread(target=doPeriodicCheck, daemon=False)
+                self._thread.start()
+            self._threadLock.release()
+
+
 class PicklePersistentKeyValueCache(PersistentKeyValueCache):
     """
     Represents a key-value cache as a dictionary which is persisted in a file using pickle
@@ -559,14 +628,17 @@ def cached(fn: Callable[[], T], picklePath, functionName=None, validityCheckFn:
         return callFnAndCacheResult()
 
 
-# TODO: I think this class deserves some documentation on the intended usage
 class PickleCached(object):
+    """
+    Function decorator for caching function results via pickle
+    """
     def __init__(self, cacheBasePath: str, filenamePrefix: str = None, filename: str = None, backend="pickle"):
         """
-
-        :param cacheBasePath:
-        :param filenamePrefix:
-        :param filename:
+        :param cacheBasePath: the directory where the pickle cache file will be stored
+        :param filenamePrefix: a prefix of the name of the cache file to be created, to which the function name and, where applicable,
+            a hash code of the function arguments will be appended and ".cache.pickle" will be appended; if None, use "" (if filename
+            has not been provided)
+        :param filename: the full file name of the cache file to be created; this is admissible only if the function has no arguments
         """
         self.filename = filename
         self.cacheBasePath = cacheBasePath
@@ -578,11 +650,23 @@ class PickleCached(object):
         else:
             self.filenamePrefix += "-"
 
-    def __call__(self, fn: Callable, *args, **kwargs):
-        if self.filename is None:
-            self.filename = self.filenamePrefix + fn.__qualname__ + ".cache.pickle"
-        picklePath = os.path.join(self.cacheBasePath,  self.filename)
-        return lambda *args, **kwargs: cached(lambda: fn(*args, **kwargs), picklePath, functionName=fn.__name__, backend=self.backend)
+    def __call__(self, fn: Callable, *_args, **_kwargs):
+
+        def wrapped(*args, **kwargs):
+            haveArgs = len(args) > 0 or len(kwargs) > 0
+            if self.filename is None:
+                filename = self.filenamePrefix + fn.__qualname__
+                if haveArgs:
+                    filename += "-" + pickleHash((args, kwargs))
+                filename += ".cache.pickle"
+            else:
+                if haveArgs:
+                    raise Exception("Function called with arguments but full cache filename specified: specify a cache filename prefix only to account for argument values")
+                filename = self.filename
+            picklePath = os.path.join(self.cacheBasePath, filename)
+            return cached(lambda: fn(*args, **kwargs), picklePath, functionName=fn.__name__, backend=self.backend)
+
+        return wrapped
 
 
 class LoadSaveInterface(ABC):
